'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function getContext() {
  var vm = this.$validator || this;
  var componentID = Object.keys(vm.context.components)[Object.keys(vm.context.components).length - 1];
  return vm.context.components[componentID];
}
function setContext() {
  this.$validator.context.components = Object.assign({}, this.$validator.context.components, _defineProperty({}, this._uid, this));
}

function alpha (value, msg) {
  return !Array.isArray(value) && !/^\s*([0-9a-zA-Z]*)\s*$/.test(value) && (msg || 'Must only contain letters and numbers');
}

function numeric (value, msg) {
  return !Array.isArray(value) && !/^[0-9]*$/.test(value) && (msg || 'Must be a numeric value');
}

function pattern (value, msg, forms, form, key) {
  var pattern = forms[form][key].pattern;
  return value && !pattern.test(value) && (msg || 'Campo inválido');
}

function required (value, msg) {
  return !value && (msg || 'Field is required');
}

function alphabetic (value, msg) {
  return !Array.isArray(value) && !/^[a-zA-Z]*$/.test(value) && (msg || 'Must be a alphabetic value');
}

function custom (value, msg, validations, form, key) {
  var callbacks = validations[form][key]['custom'];
  var valid = callbacks.every(function (f) {
    return f(value);
  });
  return (!Array.isArray(callbacks) || !valid) && (msg || 'Campo inválido');
}



var VALIDATIONS = /*#__PURE__*/Object.freeze({
  alpha: alpha,
  numeric: numeric,
  pattern: pattern,
  required: required,
  alphabetic: alphabetic,
  custom: custom
});

function getErrors(validations, messages, form, key, value) {
  var hasRule = function hasRule(rule) {
    return validations[form] && validations[form][key] && validations[form][key][rule];
  };

  var getMessage = function getMessage(rule) {
    return messages && messages[form] && messages[form][key] && messages[form][key][rule];
  };

  var getError = function getError(rule, msg) {
    return VALIDATIONS[rule](value, msg, validations, form, key);
  };

  var RULES = Object.keys(VALIDATIONS);
  var errors = [];
  RULES.some(function (rule) {
    if (hasRule(rule)) {
      var msg = getMessage(rule);
      var error = getError(rule, msg);
      if (error) errors = _toConsumableArray(errors).concat([error]);
    }
  });
  return errors;
}

function validate(form, key, value) {
  var vm = getContext.call(this);
  var validations = vm.validations;
  var messages = vm.messages;
  var errors = getErrors(validations, messages, form, key, value);
  var isTouched = validations[form] && validations[form][key] && validations[form][key].isTouched;
  validations[form][key] = Object.assign({}, validations[form][key], {
    errors: errors,
    isTouched: true,
    isDirty: !!value || isTouched,
    isFilled: !!value,
    isValid: errors.length <= 0
  });
}

var validateField = validate;

function setMessages(source, newMessages) {
  var _arr = Object.values(source);

  for (var _i = 0; _i < _arr.length; _i++) {
    var form = _arr[_i];

    var _arr2 = Object.values(form);

    for (var _i2 = 0; _i2 < _arr2.length; _i2++) {
      var input = _arr2[_i2];

      var _arr3 = Object.keys(input);

      for (var _i3 = 0; _i3 < _arr3.length; _i3++) {
        var messageKey = _arr3[_i3];

        if (newMessages[messageKey]) {
          input[messageKey] = newMessages[messageKey];
        }
      }
    }
  }
}
function setProxy() {
  var validationsProxy = new Proxy(this.validations, {
    deleteProperty: function deleteProperty(target, prop) {
      if (prop in target) {
        process.env.NODE_ENV === 'development' && console.warn("you can not remove validations property");
        return true;
      }
    }
  });
  this.validations = validationsProxy;
}

function forceValidation(form, element) {
  var key = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : element.name;
  var value = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : element.value;
  var vm = getContext.call(this);
  var isTouched = vm.validations[form] && vm.validations[form][key] && vm.validations[form][key].isTouched; // to prevent unnecessary checks

  if (vm.validations && !isTouched) {
    vm.validations[form][element.name].isTouched = true;
    validateField.call(this, form, key, value || '');
  }
}

function addTouchListener(formName, input) {
  var _this = this;

  // register events only for those who have validation
  if (this.validations[formName] && this.validations[formName][input.name]) {
    input.addEventListener('blur', function () {
      return forceValidation.call(_this, formName, input);
    }, {
      once: true
    });
  }
}

function setListenersTouch() {
  var vm = getContext.call(this); // dynamically records listeners to activate touch inputs

  vm.$nextTick(function () {
    var NodeListForms = vm.$el.querySelectorAll('form[name]');

    if (NodeListForms.length) {
      NodeListForms.forEach(function (NodeForm) {
        var formName = NodeForm.getAttribute('name');
        Array.from(NodeForm).forEach(function (input) {
          return addTouchListener.call(vm, formName, input);
        });
      });
    } else {
      console.warn('follow the instructions in the documentation to correctly register the form');
    }
  });
}

var services = /*#__PURE__*/Object.freeze({
  setMessages: setMessages,
  setProxy: setProxy,
  setListenersTouch: setListenersTouch
});

function getValidation(validation, form) {
  return !validation ? this.$options.validation : Object.assign({}, this.validations, _defineProperty({}, form, Object.assign({}, this.validations[form], validation)));
}

function watchValidate(formKey, input) {
  var _this = this;

  this.$watch(formKey.concat('.', input), function (value) {
    validateField.call(_this, formKey, input, value);
  });
}

function setValidations(validation, formKey, formValue) {
  var _this2 = this;

  // prevents unnecessary resources/loops
  var inputWithValidation = function inputWithValidation(input) {
    return Object.keys(validation[formKey] || {}).includes(input);
  };

  Object.keys(formValue).forEach(function (input) {
    if (inputWithValidation(input)) {
      watchValidate.call(_this2, formKey, input);
    }
  });
}
var defaultState = {
  isTouched: false,
  isDirty: false,
  isFilled: false,
  isValid: false,
  errors: []
};

function setFormStates(data, form, validation) {
  return _defineProperty({}, form, Object.entries(data).reduce(function (initialForm, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    var filled = {
      isFilled: !!value
    };
    var dirted = {
      isDirty: !!value
    };
    var validations = validation && validation[key];

    if (validations !== undefined) {
      initialForm[key] = Object.assign({}, defaultState, dirted, filled, validations);
    }

    return initialForm;
  }, {}));
}

function makeInitialForm(validation, formKey, formValue) {
  if (Object.keys(validation).includes(formKey)) {
    this.validations = Object.assign({}, this.validations, setFormStates(formValue, formKey, validation[formKey] || // when validation is created automatically
    validation // when validation is created dynamically
    ));
  }
}

function add(__validation, form) {
  var vm = getContext.call(this);
  var validation = getValidation.call(vm, __validation, form);
  /* eslint-disable */

  var _vm$$data = vm.$data,
      _vm$$data$validations = _vm$$data.validations,
      _vm$$data$messages = _vm$$data.messages,
      data = _objectWithoutProperties(_vm$$data, ["validations", "messages"]);
  /* eslint-enable */


  var dataWithValidation = Object.entries(data).filter(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 1),
        formKey = _ref2[0];

    return formKey === form;
  });
  dataWithValidation.forEach(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        _ = _ref4[0],
        value = _ref4[1];

    return setValidations.call(vm, validation, form, value);
  });
  makeInitialForm.call(vm, validation, form, dataWithValidation[0][1]);
  setProxy.call(vm);
}

var context = {
  components: {} // proposal....
  // let components = {}
  // export default {
  //   components,
  //   set (component, vm) {
  //     components = {
  //       ...components,
  //       components: vm
  //     }
  //   },
  //   get (component) {
  //     return components[component]
  //   }
  // }

};

var defaultStates = ['isDirty', 'isFilled', 'isTouched', 'isValid'];

function setValue(state, value) {
  if (defaultStates.includes(state)) return false;
  if (state === 'errors') return [];
  return value;
}

function resetForm(fields) {
  return Object.entries(fields).reduce(function (accFields, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        states = _ref2[1];

    accFields[key] = Object.entries(states).reduce(function (accStates, _ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          state = _ref4[0],
          value = _ref4[1];

      accStates[state] = setValue(state, value);
      return accStates;
    }, {});
    return accFields;
  }, {});
}

function reset (formToReset) {
  var vm = getContext.call(this);
  var isEnv = process.env.NODE_ENV === 'development';
  var hasForm = Object.keys(vm.validations).includes(formToReset);

  if (formToReset && !hasForm && isEnv) {
    console.warn("it was not possible to reset the ".concat(formToReset, " of a form that does not exist."));
    return;
  }

  if (!formToReset) formToReset = Object.keys(vm.validations)[0];
  vm.validations = Object.entries(vm.validations).reduce(function (accForms, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        form = _ref2[0],
        fields = _ref2[1];

    form === formToReset ? accForms[form] = resetForm(fields) : accForms[form] = fields;
    return accForms;
  }, {});
  setProxy.call(vm);
  vm.$validator.validateOnBlur && setListenersTouch.call(vm);
}

function validate$1 (form) {
  var vm = getContext.call(this);
  if (!form) form = Object.keys(vm.validations)[0];
  return new Promise(function (resolve) {
    var errors = [];
    Object.entries(vm.validations).filter(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 1),
          formName = _ref2[0];

      return formName === form;
    }).forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          _ = _ref4[0],
          states = _ref4[1];

      var _arr = Object.keys(states);

      for (var _i = 0; _i < _arr.length; _i++) {
        var key = _arr[_i];
        validateField.call(vm, form, key, vm[form][key]);
        errors.push(vm.validations[form][key].isValid);
      }
    });
    var isFormValid = errors.every(function (error) {
      return error;
    });
    resolve(isFormValid);
  });
}

function init() {
  var vm = getContext.call(this);
  var validation = vm.$options.validation;
  /* eslint-disable */

  var _vm$$data = vm.$data,
      _vm$$data$validations = _vm$$data.validations,
      _vm$$data$messages = _vm$$data.messages,
      data = _objectWithoutProperties(_vm$$data, ["validations", "messages"]);
  /* eslint-enable */
  // prevents unnecessary resources/loops


  var dataWithValidation = function dataWithValidation(_ref) {
    var _ref2 = _slicedToArray(_ref, 1),
        key = _ref2[0];

    return !!(validation[key] && Object.keys(validation[key]).length);
  };

  Object.entries(data).filter(dataWithValidation).forEach(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        formKey = _ref4[0],
        formValue = _ref4[1];

    makeInitialForm.call(vm, validation, formKey, formValue);
    setValidations.call(vm, validation, formKey, formValue);
  });
  setProxy.call(vm);
}

var snapshots = {
  components: {}
};



var prototypes = /*#__PURE__*/Object.freeze({
  add: add,
  context: context,
  reset: reset,
  validate: validate$1,
  init: init,
  snapshots: snapshots
});

var messages = function messages(options) {
  return options && options.messages && {
    messages: options.messages
  } || {};
};

var validateOnBlur = function validateOnBlur(options) {
  return options && {
    validateOnBlur: options.validateOnBlur
  };
};

var validator = {
  install: function install(Vue, options) {
    Object.defineProperty(Vue.prototype, '$validator', {
      get: function get() {
        return Object.assign({}, prototypes, services, messages(options), validateOnBlur(options));
      }
    });
  }
};

function getSnapshots() {
  var validation = this.$options.validation;
  /* eslint-disable */

  var _this$$data = this.$data,
      _this$$data$validatio = _this$$data.validations,
      _this$$data$messages = _this$$data.messages,
      data = _objectWithoutProperties(_this$$data, ["validations", "messages"]);
  /* eslint-enable */


  return Object.entries(data).reduce(function (acc, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        dataKey = _ref2[0],
        dataValue = _ref2[1];

    Object.keys(validation).forEach(function (validationKey) {
      return validationKey === dataKey && (acc[dataKey] = dataValue);
    });
    return acc;
  }, {});
}
function setSnapshot() {
  this.$validator.snapshots.components = Object.assign({}, this.$validator.snapshots.components, _defineProperty({}, this._uid, getSnapshots.call(this)));
}

var validator$1 = {
  validator: {
    bind: function bind(el, binding, vnode) {
      var vm = vnode.context;

      var _vnode$data$model$exp = vnode.data.model.expression.split('.'),
          _vnode$data$model$exp2 = _slicedToArray(_vnode$data$model$exp, 1),
          form = _vnode$data$model$exp2[0]; // if the form property does not exist in validations, set.


      if (!vm.validations[form]) {
        vm.$set.call(vnode, vm.validations, form, {});
      }
    },
    inserted: function inserted(el, _ref, vnode) {
      var rules = _ref.value;

      var _vnode$data$model$exp3 = vnode.data.model.expression.split('.'),
          _vnode$data$model$exp4 = _slicedToArray(_vnode$data$model$exp3, 2),
          form = _vnode$data$model$exp4[0],
          key = _vnode$data$model$exp4[1];

      var validations = _defineProperty({}, key, rules);

      vnode.context.$validator.add(validations, form);
    }
  }
};

var formSetup = {
  created: function created() {
    var _this$$options = this.$options,
        validation = _this$$options.validation,
        messages = _this$$options.messages;
    this.messages = messages || null;

    if (validation) {
      // set the component context values
      setContext.call(this); // set snapshot of the initial form

      setSnapshot.call(this); // overrides default messages based on global message options

      if (this.$validator.messages && this.messages && this.messages.length) {
        setMessages(this.messages, this.$validator.messages);
      } // create validation states/rules


      this.$validator.init();
      this.$validator.validateOnBlur && setListenersTouch.call(this);
    }
  },
  directives: validator$1,
  data: function data() {
    return {
      validations: {},
      messages: {}
    };
  },
  methods: {
    $hasError: function $hasError(key, form) {
      if (this.validations && Object.keys(this.validations).length) {
        // in a single-form scenario, the scope is unique, and you do not have to explicitly name the form
        if (!form) form = Object.keys(this.validations)[0];
        var input = this.validations[form] && this.validations[form][key];
        return input && input.isTouched && !input.isValid && input.errors[0];
      }

      return false;
    }
  }
};

exports.validator = validator;
exports.formSetup = formSetup;
